//
// Name        : FTCOM.PRG
// Project     : Fischer Techniek interfacing
// Description : Interface-driver for Intelligent Interface (30402) 
//                with possible extension module 16554,
//                and the RoboPro-interface in II(30402)-COM mode
// Language    : xBase++
// Author      : Rene Nooteboom
// Email:      : r.nooteboom2@chello.nl
// libs        : xbToolsIII for com- and bit manipulation functions and noot.obj
// Date created: 27-07-2005
// Date updated: 02/09/2006 22:06
// Remark      : can open and control 4 COM ft-30402-interfaces at the same time
//               all can be used&controlled with extension-slave module(16554)
//
// FUNCTION FtCAD([nComPort],[nAnalogInput],[iSleepIdleTime]) RETURN nADValue
// FUNCTION FtCExtension([nComPort],[lSet]) RETURN lOldStatus // module 16554
// FUNCTION FtCInit([nComPort],[laskvalue],[lExtensionModule]) RETURN nComPort
// FUNCTION FtCInput([nComPort],[nOutput],[iSleepIdleTime]) RETURN asc(cInputByte)
// FUNCTION FtCSensor([nComPort],nE1toE16,[iSleepIdleTime]) RETURN lSensor
// PROCEDURE FtCButton([nComPort],nE1toE16,[iSleepIdleTime])
// PROCEDURE FtCButtonPushed([nComPort],nE1toE16,[iSleepIdleTime])
// PROCEDURE FtCClose([nComPort],[iSleepIdleTime])
// Procedure FtCCloseAll([iSleepIdleTime])
// PROCEDURE FtCDelay(nDelayTime,[iSleepIdleTime])
// PROCEDURE FtCMotor([nComPort],nMotorNr,nDirection,[iSleepIdleTime]) 
// PROCEDURE FtCMotorLeft([nComPort],nMotorNr,[iSleepIdleTime])
// PROCEDURE FtCMotorOff([nComPort],nMotorNr,[iSleepIdleTime])
// PROCEDURE FtCMotorOn([nComPort],nMotorNr,[iSleepIdleTime])
// PROCEDURE FtCMotorRight([nComPort],nMotorNr,[iSleepIdleTime])
// PROCEDURE FtCMotorsOff([nComPort],[iSleepIdleTime])
// PROCEDURE FtCOffNr([nComPort],nO1toO16,[iSleepIdleTime]) // only for RoboPro interface
// PROCEDURE FtCOnNr([nComPort],nO1toO16,[iSleepIdleTime]) // only for RoboPro interface
// PROCEDURE FtCOutput([nComPort],nOutput,[iSleepIdleTime])
// Procedure FTCShowUsedIF()
//
#include "common.ch"
#include "xbp.ch"
#include "xbtcom.ch"
#include "noot.ch"
#include "ft.ch"
//
#define MAX_COM_PORT_NR 22
//
STATIC nMotorStatus_:={0,0,0,0} // max 4 comports simultaneously open
Static lExtModule_:={FALSE,FALSE,FALSE,FALSE} // is extension slave module connected&used?
STATIC nUsedComPorts_ := {}
//
FUNCTION FtCInit(nComPort,laskvalue,lExtensionModule) // initialize & open COM-port, returns -1 if error
 LOCAL cmsg,getlist:={},oCrt,oParent,cComPort:="COMxx",i,Index,cComPorts_:={}, ;
  cInputByte:=Chr(0),cCommand,lsended:=FALSE,lFtReaction:=TRUE,nExtensionModule:=0
 LOCAL cInputBytes:=Chr(0)+Chr(0),nInput2,oOwner
 Default laskvalue to FALSE,lExtensionModule to FALSE
 If IS_NIL(nComPort)
  nComPort := -1
 EndIf
 If Len(nUsedComPorts_) > 3
  ALARMTONE
  cmsg := "FtCInit():Error: Maximum 4 COM ports can be opened simultaneously"
  MsgBox(cmsg)
  Return -1
 EndIf
 If IS_ELEMENT(nUsedComPorts_,nComPort)
  ALARMTONE
  cmsg := "FtCInit():Error: COM" + str_zero(nComPort,2,0) + " already used"
  MsgBox(cmsg)
  Return -1
 EndIf
 nExtensionModule := If(lExtensionModule,1,0)
 If laskvalue .OR. nComPort < 1  // if needed ask for comport in new xbpcrt window
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},10,50,"Select Comport")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  @ MaxRow()-1,1 SAY "Searching for available COM-ports..."
  For i:=1 to MAX_COM_PORT_NR 
   If (AScan(nUsedComPorts_,i)) < 1 // port not already in use ?
    If com_open(i)  // existing port ?
     com_close(i)  
     AAdd(cComPorts_,"COM"+str_zero(i,2,0))
    EndIf
   EndIf
  Next i
  @ MaxRow()-1,1 Clear to MaxRow()-1,MaxCol()
  @ 2,2 SAY "ComPort:"
  @ 3,2 SAY "Slave module?(0=no,1=yes):"
  @ 2,30 Get cComPort VALID {|oGet|GetListBox(oGet,cComPorts_)} Color "W/B,GR+/B"
  @ 3,30 Get nExtensionModule PICTURE "9" VALID IS_BETW(nExtensionModule,0,1)
  Read
  nComPort := Val(Right(AllTrim(cComPort),2))
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf
 EndIf
 lExtensionModule := (nExtensionModule == 1)
 If nComPort > 0
  IF com_open(nComPort)
   com_hard(nComPort,FALSE) // no hardware handshake
   IF com_init(nComPort,9600,"N",8,1)
    com_dtr(nComPort,TRUE) // activate DTR signal
// com_rts(nComPort,TRUE)
// com_sflush(nComPort)
    com_flush(nComPort) // clear receive buffer
//
// check to see if there is a fischer techniek interface responding here ?
//    
    If !lExtensionModule // without extension module
     cCommand := Chr(193) + Chr(0) // 0xC1 : request for inputbyte and output zero byte
     IF lsended := (com_send(nComPort,cCommand) == 0) 
      Sleep(10) // wait 0.1sec = 10 * 1/100s
      StartStopWatch()
      Do While com_count(nComPort) < 1
       Sleep(1) // 1/100s
       If ElapsedTime() > 2
        lFtReaction := FALSE
        Exit
       EndIf
      EndDo
      If !lFtReaction
       com_close(nComPort)
       ALARMTONE
       MsgBox("FtCInit():Error: no FischerTechnik interface responding at COM" + ;
        str_zero(nComPort,2,0))
       Return -1      
      Else
       cInputByte := com_read(nComPort,1)	
      EndIf
     Else
      com_close(nComPort)
      ALARMTONE
      MsgBox("FtCInit():Error: sending 2 bytes to COM" + Str_zero(nComPort,2,0))
      Return -1     	
     EndIf 	
    Else // with extension slave module
     cCommand := Chr(194) + Chr(0) + Chr(0) // 0xC2 : request for inputbytes and output zero bytes
     IF lsended := (com_send(nComPort,cCommand) == 0) 
      Sleep(10) // wait 0.1sec = 10 * 1/100s
      StartStopWatch()
      Do While com_count(nComPort) < 2
       Sleep(1) // 1/100s
       If ElapsedTime() > 2
        lFtReaction := FALSE
        Exit
       EndIf
      EndDo  
      If !lFtReaction
       com_close(nComPort)
       ALARMTONE
       MsgBox("FtCInit():Error:no FT-interface with Extension-module responding at COM" + ;
        str_zero(nComPort,2,0))
       Return -1      
      Else
       cInputBytes := com_read(nComPort,2)
       nInput2 := Asc(SubStr(cInputBytes,2,1)) // get inputs extension interface 16554
       If nInput2 == 0xFF // all inputs high then problaby 16554 is not connected or no power	
        com_close(nComPort)
        ALARMTONE
        MsgBox("FtCInit(): Error: FtExtension-module inputs(E9-E16) all set?!, COM" + ;
         str_zero(nComPort,2,0))
        Return -1   
       EndIf
      EndIf      	
     Else
      com_close(nComPort)
      ALARMTONE
      MsgBox("FtCInit():Error: sending 3 bytes to COM" + Str_zero(nComPort,2,0))
      Return -1   	
     EndIf 
    EndIf 
    com_flush() // clear receive buffer
    AAdd(nUsedComPorts_,nComPort)
    lExtModule_[Len(nUsedComPorts_)] := lExtensionModule
    FtCOutput(nComPort,0) // switch all motors(outputs) off
    Return nComPort
   Else
    com_close(nComPort)	
    cmsg := "FtCInit():Error: failed to initialize COM" + str_zero(nComPort,2,0)
    ALARMTONE
    MsgBox(cmsg)
    Return -1
   ENDIF
  ELSE
   cmsg := "FtCInit():Error: failed to open COM" + str_zero(nComPort,2,0)
   ALARMTONE
   MsgBox(cmsg)
   Return -1
  ENDIF
 ELSE
  nComPort := -1
 ENDIF
RETURN nComPort
//
PROCEDURE FtCClose(nComPort,iSleepIdleTime)
 LOCAL indexComPort
 Default iSleepIdleTime to 1 // default 1/100s
 If Len(nUsedComPorts_) > 0
  If IS_NIL(nComport)
   nComPort := nUsedComPorts_[1] // default first used COMportnumber
  EndIf
  If nComPort > 0 .AND. IS_ELEMENT(nUsedComPorts_,nComPort)
   indexComPort := AScan(nUsedComPorts_,nComPort)
   FtCOutput(nComPort,0,iSleepIdleTime)
   com_flush(nComPort)
// com_sflush(nComPort)
   com_close(nComPort)
   ADel(nUsedComPorts_,indexComPort)
   ASHRINK(nUsedComPorts_)
   lExtModule_[indexComPort] := FALSE
  EndIf
 EndIf
Return
//
Procedure FtCCloseAll(iSleepIdleTime)
 LOCAL i
 Default iSleepIdleTime to 1
 For i:=1 to Len(nUsedComPorts_)
  FtCClose(nUsedComPorts_[i],iSleepIdleTime)
 Next i
Return 
//
// get/set if extension module is used
//
Function FtCExtension(nComPort,lOnOff)
 LOCAL indexComPort,lOldValue
 Default nComPort to nUsedComPorts_[1]
 indexComPort := AScan(nUsedComPorts_,nComPort)
 lOldValue := lExtModule_[indexComport]
 If IS_LOGIC(lOnOff)
  lExtModule_[indexComPort] := lOnOff
 EndIf
 Return lOldValue
//
//   nInput : = FtCInput(nComPort,nOutput,iSleepIdleTime)
//
//   extension(slave) interface       first(master) interface 
//           high byte                   low byte
//           0000 0000                   0000 0000
//   E16 E15 E14 E13  E12 E11 E10 E9   E8 E7 E6 E5  E4 E3 E2 E1
//                    together: nInput
//
//   nOutput :
//   
//   extension module       first interface
//   M8 M7  M6 M5            M4 M3  M2 M1
//   00 10  01 00            10 01  00 10 example
//    high byte               low byte
//    nOutput2                nOutput1
//          together : nOutput = nMotorStatus_[] = nOutput2*256 + nOutput1
//
Function FtCInput(nComPort,nOutput,iSleepIdleTime) // all optional
 LOCAL lsended:=FALSE,cCommand,cInputByte:=chr(0),cmsg,lOutput:=FALSE, ;
  indexComPort,cInputBytes:=Chr(0)+Chr(0),nInput:=0,nOutput1,nOutput2
 DEFAULT nComPort TO nUsedComPorts_[1] // default first used COMportnumber
 Default iSleepIdleTime to 1 // 1/100s=0.01s 
 indexComPort := AScan(nUsedComPorts_,nComPort)
 IF nOutput == NIL
  nOutput := nMotorStatus_[indexComPort]
 ELSE
  lOutput := TRUE
 ENDIF
 com_flush(nComPort) // clear receive buffer
// com_rts(nComPort,TRUE)
// com_dtr(nComPort,TRUE) // activate DTR signal
 If !lExtModule_[indexComPort]  // no extension slave module connected
  cCommand := chr(193) + chr(nOutput) // C1h=193d request for inputbyte
  IF lsended := (com_send(nComPort,cCommand) == 0)
   If iSleepIdleTime > 0
    DO WHILE com_count(nComPort) < 1
     sleep(iSleepIdleTime) // needed for OS processes and multi-threading ??
    EndDo
   Else
    DO WHILE com_count(nComPort) < 1
    EndDo	
   EndIf
   cInputByte := com_read(nComPort,1) // remove chars as they are read
   nInput := Asc(cInputByte)
  Else
   cmsg := "FtCInput():Error: failed to send 2 bytes to COM" + str_zero(nComPort,2,0)
   ALARMTONE
   MsgBox(cmsg)
   Return 0
  EndIf
 Else // with extension slave module used
  nOutput1 := numand(nOutput,"FF") // 0000 0000 1111 1111 get low byte = outputs first interface 
  nOutput2 := numand(nOutput,"FF00") // get high byte = outputs slave interface
  nOutput2 := nOutput2 / 256 // right shift 8 bits = 2^8 = 256
  cCommand := Chr(194) + Chr(nOutput1) + Chr(nOutput2) // C2h=194d request for inputbytes
  IF lsended := (com_send(nComPort,cCommand) == 0)
   If iSleepIdleTime > 0
    DO WHILE com_count(nComPort) < 2
     sleep(iSleepIdleTime) // needed for OS processes and multi-threading ??
    EndDo
   Else
    DO WHILE com_count(nComPort) < 2
    EndDo	
   EndIf
   cInputBytes := com_read(nComPort,2) // remove chars as they are read
   nInput := (Asc(SubStr(cInputBytes,2,1))*256) + Asc(SubStr(cInputBytes,1,1))
  Else
   cmsg := "FtCInput():Error: failed to send 3 bytes to COM" + str_zero(nComPort,2,0)
   ALARMTONE
   MsgBox(cmsg)
   Return 0	
  EndIf
 EndIf
 If lOutput
  nMotorStatus_[indexComPort] := nOutput
 EndIf
 RETURN nInput
//
PROCEDURE FtCOutput(nComPort,nOutput,iSleepIdleTime)
 FtCInput(nComPort,nOutput,iSleepIdleTime)
RETURN
//
FUNCTION FtCSensor(nComPort,nE1toE16,iSleepIdleTime)
 LOCAL nInput:=0,lSensor:=FALSE
 Default iSleepIdleTime to 1
 nInput := FtCInput(nComPort,,iSleepIdleTime)
 lSensor := isbit(nInput,nE1toE16)
RETURN lSensor
//
PROCEDURE FtCMotorsOff(nComPort,iSleepIdleTime)
 Default iSleepIdleTime to 1
 FtCOutput(nComPort,0,iSleepIdleTime)
RETURN
//
PROCEDURE FtCMotor(nComPort,nMotorNr,nDirection,iSleepIdleTime) // MON,MCW,MCCW,MOFF
 LOCAL nOutput,indexComPort
 DEFAULT nComPort TO nUsedComPorts_[1] // default eerst gebruikte COMpoortnummer 
 DEFAULT iSleepIdleTime to 1
 indexComPort := AScan(nUsedComPorts_,nComPort)
 nOutput := nMotorStatus_[indexComPort]
 DO CASE
  CASE nMotorNr == 1
   nOutput := numand(nOutput,"FFFC") // 1111 1111 1111 1100
   nOutput := numor(nOutput,nDirection) // no shift
  CASE nMotorNr == 2
   nOutput := numand(nOutput,"FFF3") // 1111 1111 1111 0011
   nOutput := numor(nOutput,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutput := numand(nOutput,"FFCF") // 1111 1111 1100 1111
   nOutput := numor(nOutput,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutput := numand(nOutput,"FF3F") // 1111 1111 0011 1111
   nOutput := numor(nOutput,nDirection*64) // left shift 6 bits
  CASE nMotorNr == 5
   nOutput := numand(nOutput,"FCFF") // 1111 1100 1111 1111
   nOutput := numor(nOutput,nDirection*256) // left shift 8 bits
  CASE nMotorNr == 6
   nOutput := numand(nOutput,"F3FF") // 1111 0011 1111 1111
   nOutput := numor(nOutput,nDirection*1024) // left shift 10 bits
  CASE nMotorNr == 7
   nOutput := numand(nOutput,"CFFF") // 1100 1111 1111 1111
   nOutput := numor(nOutput,nDirection*4096) // left shift 12 bits
  CASE nMotorNr == 8
   nOutput := numand(nOutput,"3FFF") // 0011 1111 1111 1111
   nOutput := numor(nOutput,nDirection*16384) // left shift 14 bits   
  OTHERWISE
 ENDCASE
 FtCOutput(nComPort,nOutput,iSleepIdleTime)
RETURN
//
FUNCTION FtCAD(nComPort,nAnalogInput,iSleepIdleTime) // EX = 1 , EY =2
 LOCAL cInputBytes,nADvalue:=0,cCommand:="",cmsg,lsended,nOutput, ;
  indexComPort,nOutput1,nOutput2
 DEFAULT nComPort TO nUsedComPorts_[1],nAnalogInput TO EX
 Default iSleepIdleTime to 1 // for OS processes and multithreading,i*1/100s
 indexComPort := AScan(nUsedComPorts_,nComPort)
 nOutput := nMotorStatus_[indexComPort]
 com_flush(nComPort) // clear receive buffer
 If !lExtModule_[indexComPort] // no extension module connected 
  DO CASE
   CASE nAnalogInput == EX
    cCommand := chr(197)   // 197d=C5h : give inputbyte and EX(2bytes)
   CASE nAnalogInput == EY
    cCommand := chr(201)  //  201d=C9h : give inputbyte and EY(2bytes)
  ENDCASE
 // com_dtr(nComPort,TRUE) // activate DTR signal
  cCommand += chr(nOutput)
  IF lsended := (com_send(nComPort,cCommand) == 0)
   If iSleepIdleTime > 0
    Do While com_count(nComPort) < 3
     sleep(iSleepIdleTime) // needed for OS processes and multi-threading ??
    EndDo
   Else
    Do While com_count(nComPort) < 3
    EndDo  	
   EndIf
   cInputBytes := com_read(nComPort,3)
   nADvalue := (asc(substr(cInputBytes,2,1))*256) + asc(substr(cInputBytes,3,1))
  ELSE
   cmsg := "FtCAD():Error: failed to send 2 bytes to COM" + str_zero(nComPort,2,0)
   ALARMTONE
   MsgBox(cmsg)
  EndIf
 Else  // with extension-slave module used
  DO CASE
   CASE nAnalogInput == EX
    cCommand := chr(198)   // 198d=C6h : give inputbytes and EX(2bytes)
   CASE nAnalogInput == EY
    cCommand := chr(202)  //  202d=CAh : give inputbytes and EY(2bytes)
  ENDCASE
 // com_dtr(nComPort,TRUE) // activate DTR signal
  nOutput1 := numand(nOutput,"FF") // 0000 0000 1111 1111 get low byte = outputs first interface 
  nOutput2 := numand(nOutput,"FF00") // get high byte = outputs slave interface
  nOutput2 := nOutput2 / 256 // right shift 8 bits = 2^8 = 256
  cCommand += Chr(nOutput1) + Chr(nOutput2) 
  IF lsended := (com_send(nComPort,cCommand) == 0)
   If iSleepIdleTime > 0
    Do While com_count(nComPort) < 4
     sleep(iSleepIdleTime) // needed for OS processes and multi-threading ??
    EndDo
   Else
    Do While com_count(nComPort) < 4
    EndDo  	
   EndIf
   cInputBytes := com_read(nComPort,4)
   nADvalue := (asc(substr(cInputBytes,3,1))*256) + asc(substr(cInputBytes,4,1))
  ELSE
   cmsg := "FtCAD():Error: failed to send 3 bytes to COM" + str_zero(nComPort,2,0)
   ALARMTONE
   MsgBox(cmsg)
  EndIf  	
 EndIf
 Return nADvalue
//
PROCEDURE FtCButton(nComPort,nE1toE16,iSleepIdleTime)
 Default iSleepIdleTime to 1
 If iSleepIdleTime > 0
  DO WHILE !FtCSensor(nComPort,nE1toE16,iSleepIdleTime)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !FtCSensor(nComPort,nE1toE16,0)
  EndDo
 EndIf
RETURN
//
PROCEDURE FtCButtonPushed(nComPort,nE1toE16,iSleepIdleTime)
 Default iSleepIdleTime to 1
 If iSleepIdleTime > 0
  DO WHILE FtCSensor(nComPort,nE1toE16,iSleepIdleTime)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE FtCSensor(nComPort,nE1toE16,0)
  EndDo
 EndIf
Return
//
PROCEDURE FtCOnNr(nComPort,nO1toO16,iSleepIdleTime) // only for RoboPro interface
 LOCAL nOutput,indexComPort
 Default nComPort TO 1,iSleepIdleTime to 1
 indexComPort := AScan(nUsedComPorts_,nComPort) 
 nOutput := nMotorStatus_[indexComPort] 
 nOutput := setbit(nOutput,nO1toO16)
 FtCOutput(nComPort,nOutput,iSleepIdleTime)
RETURN
//
PROCEDURE FtCOffNr(nComPort,nO1toO16,iSleepIdleTime) // only for RoboPro interface
 LOCAL nOutput,indexComPort
 Default nComPort TO 1,iSleepIdleTime to 1
 indexComPort := AScan(nUsedComPorts_,nComPort) 
 nOutput := nMotorStatus_[indexComPort]
 nOutput := clearbit(nOutput,nO1toO16)
 FtCOutput(nComPort,nOutput,iSleepIdleTime)
RETURN
//
PROCEDURE FtCMotorLeft(nComPort,nMotorNr,iSleepIdleTime)
 Default iSleepIdleTime to 1
 FtCMotor(nComPort,nMotorNr,MCCW,iSleepIdleTime)
RETURN
//
PROCEDURE FtCMotorRight(nComPort,nMotorNr,iSleepIdleTime)
 Default iSleepIdleTime to 1
 FtCMotor(nComPort,nMotorNr,MCW,iSleepIdleTime)
RETURN
//
PROCEDURE FtCMotorOff(nComPort,nMotorNr,iSleepIdleTime)
 Default iSleepIdleTime to 1
 FtCMotor(nComPort,nMotorNr,MOFF,iSleepIdleTime)
RETURN
//
PROCEDURE FtCMotorOn(nComPort,nMotorNr,iSleepIdleTime)
 Default iSleepIdleTime to 1
 FtCMotor(nComPort,nMotorNr,MON,iSleepIdleTime)
RETURN
//
// sec delay but keeps interfaces alive
//
PROCEDURE FtCDelay(nDelayTime,iSleepIdleTime)
 LOCAL nBeginTime := seconds(),i
 Default iSleepIdleTime to 1 // 1/100s
 DO WHILE TRUE
  IF nBeginTime > seconds()
   nBeginTime -= 86400
  EndIf
  If iSleepIdleTime > 0
   FOR i:=1 TO Len(nUsedComPorts_)
    FtCInput(nUsedComPorts_[i],,iSleepIdleTime)
   NEXT i
  Else
   FOR i:=1 TO Len(nUsedComPorts_)
    FtCInput(nUsedComPorts_[i],,0)
   NEXT i
  EndIf
  If iSleepIdleTime > 0
   Sleep(iSleepIdleTime)
  EndIf
  IF (seconds() - nBeginTime) > nDelayTime
   EXIT
  EndIf
 ENDDO
Return
//
Procedure FTCShowUsedIF()
 LOCAL cmsg:="FT interface uses:",i
 FOR i := 1 TO Len(nUsedComPorts_)
  cmsg += " COM" + str_zero(nUsedComPorts_[i],2,0) + If(lExtModule_[i],"(+Slave)","") 
 Next i
 MsgBox(cmsg)
RETURN
//
// end of ftcom.prg
//