//
// Name: FTLPT.PRG
// Project: Fischer Technik interfacing
// Description: ft lpt 30520 interface driver
// Language: xBase++
// Libs : needs inpout32.dll (see io.prg(.obj))
// Author: Rene Nooteboom
// Email: r.nooteboom2@chello.nl
// Date created: 21-08-2005
// Date updated: 02/09/2006 21:53
//
//  default values :
//
//  parallel port  I/O   IRQ
//
//   LPT1         378H    7
//   LPT2         278H    5
//   LPT3         3BCH    -
//   LPT4         2BCH    -
//
//  example ftlpt.ini file :  //
//                            LPT3_IO = h378
//                            LPT3_DELAY = 2
//                            LPT2_IO = 632
//                            LPT2_ANALOG = 7
//                            //
//
// FUNCTION FtLAD(nLptPort,nAnalogInput) RETURN nAnalogValue
// FUNCTION FtLHexIO(nLptPort) RETURN cHexLPTIOAddress
// FUNCTION FtLInit(nLptPort,lAskValues,xIOAddress,nLptDelay,nAnalogScale)
//  RETURN nLptPort
// FUNCTION FtLInput(nLptPort) RETURN nInputByte
// FUNCTION FtLSensor(nLptPort,nE1toE8) RETURN lSensor
// PROCEDURE FtLButton(nLptPort,nE1toE8,iSleepIdleTime)
// PROCEDURE FtLButtonPushed(nLptPort,nE1toE8,iSleepIdleTime)
// PROCEDURE FtLClose(nLptPort)
// Procedure FtLCloseAll()
// PROCEDURE FtLDelay(nDelayTime,iSleepIdleTime)
// PROCEDURE FtLMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
// PROCEDURE FtLMotorLeft(nLptPort,nMotorNr)
// PROCEDURE FtLMotorOff(nLptPort,nMotorNr)
// PROCEDURE FtLMotorOn(nLptPort,nMotorNr)
// PROCEDURE FtLMotorRight(nLptPort,nMotorNr)
// PROCEDURE FtLMotorsOff(nLptPort)
// PROCEDURE FtLOutput(nLptPort,nByte)
// Procedure FtLShowUsedIF()
//
#include "common.ch"
#include "xbp.ch"
#include "noot.ch"
#include "ft.ch"
#include "ftlpt.ch"
//
STATIC cHexLptIO_ := {"378","278","3BC","2BC"} // max 4 lptports
STATIC nLptBasisReg_ := {888,632,956,700} 
STATIC nLptStatusReg_ := {889,633,957,701} // offset 1 from dataregister
STATIC nLptControlReg_ := {890,634,958,702} // offset 2 from dataregister
STATIC lLptUsed_ := {FALSE,FALSE,FALSE,FALSE},nMotorByte_:={0,0,0,0}
Static nLptDelay_ := {10,10,10,10} // minimum output loops is 1 see WriteToPort()
Static nAn_Scale_ := {5,5,5,5} // analog scaling , default is 5
//
FUNCTION FtLInit(nLptPort,lAskValues,xIOAddress,nLptDelay,nAnalogScale) 
// 
// default nLptDelay is 10 loops
// default nAnalogScale is 5
// all arguments optional,xIOAddress can be numeric decimal or a hex-string
//
 LOCAL GetList:={},nStatusRegIO,nControlRegIO,nBasisRegIO,oCrt,oParent,oOwner
 DEFAULT nLptPort TO 1,lAskValues TO TRUE
 IF xIOAddress == NIL
  xIOAddress := FtLGetIOAddress(nLptPort)
 ENDIF
 IF IS_NUM(xIOAddress) // convert to hex
  xIOAddress := padl(alltrim(ntoc(xIOAddress,16,3)),3,"0")
 EndIf
 If empty(nLptDelay)
  nLptDelay := FtLGetLptDelay(nLptPort)
 EndIf
 If empty(nAnalogScale)
  nAnalogScale := FtLGetAnalogScale(nLptPort)
 EndIf
 IF lAskValues
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},8,38,"Select LPT-port")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  Set Cursor on
  @ 2,6 SAY "LPT-port(1 to 4)  :"
  @ 3,6 SAY "IO-Address(in hex):"
  @ 4,6 SAY "Lpt Delay loops:"
  @ 5,6 SAY "Analog Scaling:"
  @ 2,30 GET nLptPort PICTURE "9" VALID IS_BETW(nLptPort,1,4)
  @ 3,30 GET xIOAddress PICTURE "@! XXX" VALID IS_HEX(xIOAddress)
  @ 4,30 Get nLptDelay PICTURE "999" VALID nLptDelay > 0
  @ 5,30 Get nAnalogScale PICTURE "999" VALID nAnalogScale > 0
  Read
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf  
 EndIf
 If nLptPort > 0
  If nLptPort > 4
   ALARMTONE
   MsgBox("FtLInit():Error: LPT<x> must be 1-4")
   Return -1
  EndIf
  If lLptUsed_[nLptPort]
   ALARMTONE
   MsgBox("FtLinit():Error: LPT" + Str(nLptPort,1,0) + " already in use")
   Return -1
  EndIf
  xIOAddress := zeropad(AllTrim(xIOAddress),3)
  nBasisRegIO := cton(xIOAddress,16)
  nStatusRegIO := nBasisRegIO + 1
  nControlRegIO := nBasisRegIO + 2
  If GetIODllHandle() == 0
   LoadIODll()
  EndIf
  If !(GetIODllHandle() == 0)  
   cHexLptIO_[nLptPort] := xIOAddress
   nLptBasisReg_[nLptPort] := nBasisRegIO
   nLptStatusReg_[nLptPort] := nStatusRegIO
   nLptControlReg_[nLptPort] := nControlRegIO   
   If !empty(nLptDelay)
    nLptDelay_[nLptPort] := nLptDelay // nr of output loops to lptport see writetoport()
   EndIf
   If !empty(nAnalogScale)
    nAn_Scale_[nLptPort] := nAnalogScale
   EndIf
   lLptUsed_[nLptPort] := TRUE
   FtLOutput(nLptPort,0) // switch all motors off
  Else
   ALARMTONE
   MsgBox("FtLInit():Error: loading the IO Dll")
   nLptPort := -1
  EndIf
 Else
  nLptPort := -1
 EndIf
RETURN nLptPort
//
PROCEDURE FtLClose(nLptPort)
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  FtLOutput(nLptPort,0)
  lLptUsed_[nLptPort] := FALSE
 ENDIF
Return
//
Procedure FtLCloseAll()
 LOCAL i
 For i:=1 to 4
  FtLClose(i)
 Next i
 CloseIODll()
 Return 
//
Static Procedure WriteToPort(nLptPort,xValue)
 LOCAL i
 For i:=1 to nLptDelay_[nLptPort]
  outbyte(cHexLptIO_[nLptPort],xValue)
 Next i
 Return
// 
Static Function ReadPort(nLptPort)
 LOCAL nByte
 nByte := inbyte(nLptStatusReg_[nLptPort])
 Return nByte
//
FUNCTION FtLInput(nLptPort)
 LOCAL nBitNr,nTestByte,nInputByte:=0
 DEFAULT nLptPort TO 1
 WriteToPort(nLptPort,"32") // 0011 0010 LOAD_IN
 WriteToPort(nLptPort,"3A") // 0011 1010 LOAD & CLOCK 
 // create input byte first bit is E8,then E7....E1
 FOR nBitNr := 8 TO 1 STEP -1
  nTestByte := ReadPort(nLptPort) // read status register
  IF isbit(nTestByte,DATA_IN) // test bit DATA_IN line
   nInputByte := setbit(nInputByte,nBitNr)
  EndIf
  WriteToPort(nLptPort,"30") // 0011 0000 null
  WriteToPort(nLptPort,"38") // 0011 1000 CLOCK  
 Next nBitNr
 Return nInputByte
//
PROCEDURE FtLOutput(nLptPort,nByte)
 LOCAL nBitNr
 DEFAULT nLptport TO 1
 FOR nBitNr := 8 TO 1 STEP -1  // MSB(M4) comes first !
  IF isbit(nByte,nBitNr)
   WriteToPort(nLptPort,"34") // 0011 0100 // DATA_OUT
   WriteToPort(nLptPort,"3C") // 0011 1100 // DATA_OUT + CLOCK
  Else
   WriteToPort(nLptPort,"30") // 0011 0000 // 0
   WriteToPort(nLptPort,"38") // 0011 1000 // 0 + CLOCK	
  EndIf
 Next nBitNr
 WriteToPort(nLptPort,"39") // 0011 1001 // LOAD_OUT + CLOCK 
 nMotorByte_[nLptport] := nByte
 Return
//
// AD-waarde hangt af van speed processor
// hoe sneller de cpu hoe hoger de nAnalogValue
// dus altijd per toepassing yken
//
FUNCTION FtLAD(nLptPort,nAnalogInput) // EX=1,EY=2
 LOCAL nAnalogValue:=0,lCountIn,i:=0
 DEFAULT nLptPort TO 1,nAnalogInput TO EX
 IF nAnalogInput == EY
  WriteToPort(nLptPort,"90") // 1001 0000
 Else
  WriteToPort(nLptPort,"A0") // 1010 0000
 EndIf
 WriteToPort(nLptPort,"38") // 0011 1000 CLOCK
 lCountIn := TRUE
 DO WHILE lCountIn
  i++
  If i >= nAn_Scale_[nLptPort]
   nAnalogValue++
   i := 0   
   If nAnalogValue >= 2048
    Exit
   EndIf
  EndIf
  lCountIn := isbit(ReadPort(nLptPort),COUNT_IN)
 ENDDO
 Return nAnalogValue
//
FUNCTION FtLSensor(nLptPort,nE1toE8)
 LOCAL nInput:=0,lSensor:=FALSE
 nInput := FtLInput(nLptPort)
 lSensor := isbit(nInput,nE1toE8)
RETURN lSensor
//
PROCEDURE FtLMotorsOff(nLptPort)
 FtLOutput(nLptPort,0)
RETURN
//
PROCEDURE FtLMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nOutputByte
 DEFAULT nLptPort TO 1
 nOutputByte := nMotorByte_[nLptPort]
 DO CASE
  CASE nMotorNr == 1
   nOutputByte := numand(nOutputByte,"FC") // 1111 1100
   nOutputByte := numor(nOutputByte,nDirection) // no shift
  CASE nMotorNr == 2
   nOutputByte := numand(nOutputByte,"F3") // 1111 0011
   nOutputByte := numor(nOutputByte,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutputByte := numand(nOutputByte,"CF") // 1100 1111
   nOutputByte := numor(nOutputByte,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutputByte := numand(nOutputByte,"3F") // 0011 1111
   nOutputByte := numor(nOutputByte,nDirection*64) // left shift 6 bits
  OTHERWISE
 ENDCASE
 FtLOutput(nLptPort,nOutputByte)
RETURN
//
PROCEDURE FtLButton(nLptPort,nE1toE8,iSleepIdleTime)
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE !FtLSensor(nLptPort,nE1toE8)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !FtLSensor(nLptPort,nE1toE8)
  EndDo	
 EndIf
RETURN
//
PROCEDURE FtLButtonPushed(nLptPort,nE1toE8,iSleepIdleTime)
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE FtLSensor(nLptPort,nE1toE8)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE FtLSensor(nLptPort,nE1toE8)
  EndDo	
 EndIf
Return
//
PROCEDURE FtLMotorLeft(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MCCW)
RETURN
//
PROCEDURE FtLMotorRight(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MCW)
RETURN
//
PROCEDURE FtLMotorOff(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MOFF)
RETURN
//
PROCEDURE FtLMotorOn(nLptPort,nMotorNr)
 FtLMotor(nLptPort,nMotorNr,MON)
RETURN
//
STATIC FUNCTION FtLGetIOAddress(nLptPort)
 LOCAL xIOAddress:=NIL,csearch
 DEFAULT nLptPort TO 1
 IF file("ftlpt.ini")
  readinival("ftlpt.ini")
  csearch := "LPT" + str(nLptPort,1,0) + "_IO" // example: LPT3_IO = h3BC
  IF ini_exist(csearch)
   IF !empty(xIOAddress := ini_value(csearch))
    DO CASE
     CASE IS_NUM(xIOAddress)  // assume decimal number
     CASE IS_CHAR(xIOAddress)
      IF upper(left(xIOAddress,1)) == "H" // hex number follows "Hxxx"
       xIOAddress := padl(alltrim(substr(xIOAddress,2)),3,"0")
      ELSE
       IF !IS_HEX(xIOAddress)
        xIOAddress := NIL
       ELSE
        xIOAddress := padl(alltrim(xIOAddress),3,"0")
       ENDIF
      ENDIF
     OTHERWISE
      xIOAddress := NIL
    ENDCASE
   ENDIF
  ENDIF
  clearinival()
 ENDIF
 IF empty(xIOAddress)
  DO CASE
   CASE nLptPort == 1
    xIOAddress := "378"
   CASE nLptPort == 2
    xIOAddress := "278"
   CASE nLptPort == 3
    xIOAddress := "3BC"
   CASE nLptPort == 4
    xIOAddress := "2BC"
   OTHERWISE
    xIOAddress := "378"
  ENDCASE
 ENDIF
RETURN xIOAddress
//
Static Function FtLGetLptDelay(nLptPort)
 LOCAL cSearch,nLptDelay:=10
 Default nLptPort to 1
 If File("ftlpt.ini")
  readinival("ftlpt.ini")
  cSearch := "LPT" + Str(nLptPort,1,0) + "_DELAY"
  If empty(nLptDelay := ini_value(cSearch))
   nLptDelay := 10
  EndIf
 EndIf
Return nLptDelay
//
Static Function FtLGetAnalogScale(nLptPort)
 LOCAL cSearch,nAnalogScale:=5
 Default nLptPort to 1
 If File("ftlpt.ini")
  readinival("ftlpt.ini")
  cSearch := "LPT" + Str(nLptPort,1,0) + "_ANALOG"
  If empty(nAnalogScale := ini_value(cSearch))
   nAnalogScale := 5
  EndIf
 EndIf
Return nAnalogScale
//
Procedure FtLShowUsedIF()
 LOCAL cmsg:="Ft interface uses:",i
 FOR i := 1 TO 4
  IF lLptUsed_[i]
   cmsg += " LPT" + str(i,1,0) + ", IO:" + cHexLptIO_[i] + "h"
  ENDIF
 Next i
 MsgBox(cmsg)
Return
//
// sec delay but keeps interfaces alive
//
PROCEDURE FtLDelay(nDelayTime,iSleepIdleTime)
 LOCAL nBeginTime := seconds(),i
 DO WHILE TRUE
  IF nBeginTime > seconds()
   nBeginTime -= 86400
  ENDIF
  FOR i:=1 TO 4
   IF lLptUsed_[i]
    FtLInput(i)
   ENDIF
  NEXT i
  IF (seconds() - nBeginTime) > nDelayTime
   EXIT
  EndIf
  If iSleepIdleTime > 0
   Sleep(iSleepIdleTime)
  EndIf
 ENDDO
RETURN
//
FUNCTION FtLHexIO(nLptPort)
 LOCAL cHexIO:="---"
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  cHexIO := cHexLptIO_[nLptPort]
 ENDIF
RETURN cHexIO
//
// einde FtLpt.prg
//
