//
// Name: FT30520.PRG
// Project: Fischer Technik interfacing
// Description: ft lpt 30520 interface driver
// Language: xBase++
// Libs : based on umFish30.dll and possible LPT-kernel0 driver :
//   winRT.sys(WINNT),wrtdev0.vxd(w9x)
// Author: Rene Nooteboom
// Email: r.nooteboom2@chello.nl
// Date created: 10/05/2006 10:29
// Date updated: 02/09/2006 22:45
// Remark : only for one LPT-interface simulteanously with a possible slave interface
//          max lptportnr is 3 (1-3)
// 
// For WINNT-family OS's : depends on kernel0 parallel port driver : winRT.sys : device : "LPT"
// For W9x-family OS's   : optional kernel0 parallel port driver : wrtDev0.vxd : device : "LPT"
//                         sometimes direct IO under w9x also possible : device : "LPT1","LPT2","LPT3"
//
// Registry details for LPT-kernel0 driver :
//
// Installing for WinNT/2000/XP :
//
// 1. Copy WinRT.SYS to %SystemRoot%\system32\Driver 
// 2. Click on WinRTSYS.REG for register it.
//                 (For modifications : Open with NotePad)
// 3. Booten
//
// Installing for Win95/98/Me :
//
// 1. Copy WRTdev0.VXD to %SystemRoot%\System\VMM32
// 2. Click an WinRTVXD.REG for register it.
//                  (For modifications : Open with NotePad)
// 3. Booten
//
// Connection to the Interface : umOpenInterface(ft,"LPT")
//   (for direct IO (no kernel0 driver): use umOpenInterface(ft,"LPT1"|"LPT2"|"LPT3") )
//
// The actual Regitry entries are used.
//
// Hint : Using to parallel Interfaces with generic kernel0-LPT
// Start with OpenInterface("LPT") in addition the parameter LPTDelay must
// set to 2 for with slave ? (before OpenInterface or with OpenInterface)
//
// Registry keys for WINNT family Os : WinRTSYS.REG :
//
// [HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\EventLog\System\WinRT]
//  "EventMessageFile"="%SystemRoot%\\System32\\IoLogMsg.dll;%SystemRoot%\\System32\\Drivers\\WinRT.sys"
//  "TypesSupported"=dword:00000007
//
// [HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\WinRT]
//  "Type"=dword:00000001
//  "Group"="WinRT"
//  "ErrorControl"=dword:00000001
//  "Start"=dword:00000001 (00000001=System start,00000002=Auto start,00000003=Manual start,0=none)
//
// [HKLM\System\CurrentControlSet\Services\WinRT\WinRTdev0\Parameters\Section0]
//  "AllowConflict"=dword:1 (Printer driver may proceed, but not printing)
//  "BusType"=dword:1 (1:ISA-Karte, 0:Onboard LPT, 2:EISA, 5:PCI-card)
//  "IoPortAddress"=dword:378 ( LPT1 = 378h, LPT2 = 278h , LPT3 = 3BC)
//  "IoPortCount"=dword:3
//
// Registry keys for W9x family Os : WinRTVxD.REG
//
// [HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\VxD\WRTdev0]
//  "Start"=dword:00000002 (00000001=System start,00000002=Auto start,00000003=Manual start,0=none)
//  "StaticVxD"="*WRTDev0"   ( with w9x : auto start : 00000002 only !?)
//
// [HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\Class\WinRT\WinRTdev0\Parameters\Section0]
//  "AllowConflict"=dword:1
//  "BusType"=dword:1 (0=onboard lpt,1=isa card,2=eisa,3=pci)
//  "IoPortAddress"=dword:378 ( lpt1=378,lpt2=278,lpt3=3bc)
//  "IoPortCount"=dword:3
//
//  default values : LPT-ports
//
//  parallel port  I/O   IRQ
//
//   LPT1         378H    7
//   LPT2         278H    5
//   LPT3         3BCH    -
//
//
// Function FtLAD(nAnalogInput) // EX=1,EY=2
// Function FtLADDirect(nAnalogInput) // EX=1,EY=2
// Function FTLAnalogScale()
// Function FTLAnalogScan()
// Function FTLExtension()
// Function FtLGetCounter(nE1toE16)
// Function FtLGetModeStatus(nMotorNr)
// Function FtLGetMotors()
// Function FtLInit(nLptPort,lAskValues,nPollInterval,lWithSlave,lAnalogScan, 
//                   nLptDelay,nAnalogScale)
// Function FtLInput()
// Function FTLLoopDelay()
// Function FTLLptDelay() == FTLLoopDelay()
// Function FtLMotor(nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
// Function FtLMotorLeft(nMotorNr,nSpeed)
// Function FtLMotorOff(nMotorNr)
// Function FtLMotorOn(nMotorNr,nSpeed)
// Function FtLMotorRight(nMotorNr,nSpeed)
// Function FtLMotorSpeed(nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
// Function FtLOnNr(nO1toO16)
// Function FtLOffNr(nO1toO16)
// Function FtLOutput(nOutput)
// Function FtLOutput(nOutput)
// Function FTLPollInterval() // in ms
// Function FTLRobMotor(nMotorNr,nDirection,nSpeed,nCount)
// Function FTLRobMotors(nMotorStatus,nSpeedStatus,nModeStatus)
// FUNCTION FtLSensor(nE1toE16)
// Function FTLSetLamp(nLampNr,lOnOff)
// Function FtLSetMotors(nMotorStatus) 
// Function FtLSetMotorsEx(nMotorStatus,nSpeedStatus) 
// Function FTLSlave()
// Function FtLUsedLptPort()
// PROCEDURE FtLButton(nE1toE16,iSleepIdleTime) // wait for high
// PROCEDURE FtLButtonPushed(nE1toE16,iSleepIdleTime) // wait for low
// Procedure FtLClearCounters()
// PROCEDURE FtLClose()
// Procedure FtLCloseAll()
// PROCEDURE FtLMotorsOff()
// Procedure FtLSetCounter(nE1toE16,nValue)
// Procedure FtLSetModeStatus(nMotorNr,nMode) 
// Procedure FtLShowUsedIF()
//
#include "common.ch"
#include "inkey.ch"
#include "dll.ch"
#include "xbp.ch"
#include "os.ch"
#include "noot.ch"
#include "ft.ch"
#include "umfish30.ch"
#include "ft30520.ch"
//
Static iPortHandle:=0,nUmFish30Handle:=0,cLPT:="LPT"
//
Function FtLInit(nLptPort,lAskValues,nPollInterval,lWithSlave,lAnalogScan, ; 
 nLptDelay,nAnalogScale) // all optional
// 
// return -1 if error else nLptPort :
//
// nLptPort : 0 : generic "LPT" kernel0 device , default , see winrt.sys(WINNT) and wrtdev0.vxd(w9x)
//            1 : "LPT1",378h : direct (only w9x)
//            2 : "LPT2",278h : direct (only w9x)
//            3 : "LPT3",2BCh : direct (only w9x)
//
// nPollInterval : in mS : if ==0 then determined by OpenInterface():
//  if lAnalogScan=TRUE then nPollInterval = 100ms because AD takes time!
//     lAnalogScan=FALSE then nPollInterval = 1mS
//  my default = 10mS !
// nLptDelay     : nr of repeated outputloops :
//                 if ==0 determined by OpenInterface():default=10
//                 my default : 2
// nAnalogScale  : if ==0 determined by OpenInterface() default=5
//
 LOCAL lWinNT_Family,nAnalogScan,nSlave,cLPT_,nPortNr,oOwner ;
  ,GetList:={},oCrt,oParent,lExit:=FALSE,nStartWinRTsys:=0
 Default nLptPort TO 0,lAskValues TO TRUE
 Default nPollInterval to 10,lWithSlave to FALSE
 Default lAnalogScan to FALSE,nLptDelay to 2,nAnalogScale to 5
 If UmFish30Handle() == 0  // umfish30.dll not yet loaded ?
  If !LoadUmFish30()
   Return -1
  EndIf
 EndIf
 nUmFish30Handle := UmFish30Handle()
 lWinNT_Family := (Os(OS_FAMILY) == "WINNT")
 If lWinNT_Family
  cLPT := "LPT"
  nLptPort := 0 
 Else // w9x-family os
  If nLptPort == 0
   cLPT := "LPT"
  Else
   If IS_BETW(nLptPort,1,3)
    cLPT := "LPT" + Str(nLptPort,1,0)
   Else
    cLPT := "LPT"
    nLptPort := 0
   EndIf
  EndIf
 EndIf
 nSlave := If(lWithSlave,1,0)
 nAnalogScan := If(lAnalogScan,1,0)
 IF lAskValues
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},9,70,"LPT-Interface Configuration")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  Set Cursor on
  If !lWinNT_Family // w9x-family os
   cLPT_ := {"LPT  via  wrtdev0.vxd","LPT1(direct IO: 378h)", ;
    "LPT2(direct IO: 278h)","LPT3(direct IO: 2BCh)"} // all option string lengths= 21
   cLPT := PadR("Choose LPT-IO method",21)
   @ 1,6 SAY "LPT-device:"
   @ 1,22 Get cLPT VALID {|oGet|GetListBox(oGet,cLPT_)} Color "W/B,GR+/B"
  Else // winnt os-family
   @ 1,6 SAY "Must Use : LPT -kernel0 winRT.sys driver for WINNT-family OS" 
   @ 2,6 SAY "(re)Start WinRT.Sys? (0=no,1=yes):"
   @ 2,50 Get nStartWinRTsys PICTURE "9" VALID IS_BETW(nStartWinRTsys,0,1)
  EndIf 
  @ 3,6 SAY "Polling Interval(mS,0=auto,try:10ms):"
  @ 4,6 SAY "Analog scan(1=yes,0=no):"
  @ 5,6 SAY "Slave Interface(1=yes,0=no):"
  @ 6,6 SAY "LPT Loops OutputDelay(0=auto,try:2):"   
  @ 7,6 SAY "Analog Loops Delay(0=auto,try:5):"
  @ 3,50 GET nPollInterval PICTURE "9999" VALID nPollInterval >= 0
  @ 4,50 Get nAnalogScan PICTURE "9" VALID IS_BETW(nAnalogScan,0,1)
  @ 5,50 Get nSlave PICTURE "9" VALID IS_BETW(nSlave,0,1)  
  @ 6,50 GET nLptDelay PICTURE "999" VALID nLptDelay >= 0
  @ 7,50 GET nAnalogScale PICTURE "999" VALID nAnalogScale >= 0 
  Read
  lExit := (LastKey() == K_ESC)
  If !lWinNT_Family
   cLPT := Upper(AllTrim(Left(cLPT,4)))
  EndIf
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf  
  If lExit
   cLPT := "LPT"
   Return -1
  EndIf
 EndIf
 Do Case // nPortNr is parameter for csOpenInterfaceEx()
  Case cLPT == "LPT"
   nPortNr := 0
   nLptPort := 0
  Case cLPT == "LPT1"
   nPortNr := 9
   nLptPort := 1
  Case cLPT == "LPT2"
   nPortNr := 10
   nLptport := 2
  Case cLPT == "LPT3"
   nPortNr := 11
   nLptPort := 3  
  Otherwise
   nPortNr := 0
   cLPT := "LPT"
   nLptport := 0
 EndCase
 If lWinNT_Family .AND. (nStartWinRTsys == 1)
  StartWinRTsys()
 EndIf
 iPortHandle := DllCall(nUmFish30Handle,DLL_STDCALL,"csOpenInterfaceEx", ;
  nPortNr,nAnalogScan,nSlave,nPollInterval,nAnalogScale,nLptDelay)
 If iPortHandle == ftiError
  ALARMTONE
  MsgBox("FtLInit(): Error : csOpenInterfaceEx()")
  Return -1
 EndIf 
 FtLOutput(0)
 Return nLptPort // -1 if error
//
PROCEDURE FtLClose()
 If (iPortHandle <> 0) .AND. (iPortHandle <> ftiError)
  FtLMotorsOff()
  DllCall(nUmFish30Handle,DLL_STDCALL,"csCloseInterface",iPortHandle)
  iPortHandle := 0
 EndIf
 iPortHandle := 0
 Return
//
Procedure FtLCloseAll()
 FtLClose()
 Return
//
Function FtLInput()
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetInputs", ;
  iPortHandle) 
 Return nInputStatus
//
Function FtLOutput(nOutput)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotors",iPortHandle,nOutput)
 Return nInputStatus
//
Function FtLAD(nAnalogInput) // EX=1,EY=2
 LOCAL nAnalogValue:=0
 --nAnalogInput
 nAnalogValue := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetAnalog", ;
  iPortHandle,nAnalogInput)
 Return nAnalogValue
//
Function FtLADDirect(nAnalogInput) // EX=1,EY=2
 LOCAL nAnalogValue:=0
 --nAnaloginput
 nAnalogValue := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetAnalogDirect", ;
  iPortHandle,nAnalogInput)
 Return nAnalogValue
//
FUNCTION FtLSensor(nE1toE16)
 LOCAL lSensor
 lSensor := (DllCall(nUmFish30Handle,DLL_STDCALL,"csGetInput", ;
  iPortHandle,nE1toE16) == 1)
RETURN lSensor
//
PROCEDURE FtLMotorsOff()
 FtLOutput(0)
RETURN
//
Procedure FtLSetCounter(nE1toE16,nValue)
 DllCall(nUmFish30Handle,DLL_STDCALL,"csSetCounter",iPortHandle,nE1toE16, ;
  nValue)
Return  
//
Procedure FtLClearCounters()
 DllCall(nUmFish30Handle,DLL_STDCALL,"csClearCounters",iPortHandle)
Return  
//
Function FtLGetCounter(nE1toE16)
 LOCAL nCount:=0
 nCount := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetCounter",iPortHandle, ;
  nE1toE16)
Return nCount
//
PROCEDURE FtLButton(nE1toE16,iSleepIdleTime) // wait for high
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE !FtLSensor(nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !FtLSensor(nE1toE16)
  EndDo	
 EndIf
RETURN
//
PROCEDURE FtLButtonPushed(nE1toE16,iSleepIdleTime) // wait for low
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE FtLSensor(nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE FtLSensor(nE1toE16)
  EndDo	
 EndIf
Return
//
Function FtLMotorSpeed(nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
 LOCAL nInputStatus:=0
 Default nSpeed to FULL_MOTOR_SPEED
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotorEx", ;
  iPortHandle,nMotorNr,nDirection,nSpeed)
 Return nInputStatus
//
Function FtLMotor(nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotor", ;
  iPortHandle,nMotorNr,nDirection)
 Return nInputStatus
//
Function FtLGetMotors()
 LOCAL nOutputStatus:=0
 nOutputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetMotors", ;
  iPortHandle) 
 Return nOutputStatus
//
Function FtLGetModeStatus(nMotorNr)
 LOCAL nMode:=0
 nMode := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetModeStatus", ;
  iPortHandle,nMotorNr) 
 Return nMode
//
Procedure FtLSetModeStatus(nMotorNr,nMode) 
  DllCall(nUmFish30Handle,DLL_STDCALL,"csSetModeStatus", ;
  iPortHandle,nMotorNr,nMode)
 Return 
//
Function FtLSetMotors(nMotorStatus) 
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotors", ;
  iPortHandle,nMotorStatus)
 Return nInputStatus
//
Function FtLSetMotorsEx(nMotorStatus,nSpeedStatus) 
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetMotorsEx", ;
  iPortHandle,nMotorStatus,nSpeedStatus)
 Return nInputStatus
//
Function FtLMotorLeft(nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtLMotorSpeed(nMotorNr,ftiLeft,nSpeed)
Return nInputStatus
//
Function FtLMotorRight(nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtLMotorSpeed(nMotorNr,ftiRight,nSpeed)
Return nInputStatus
//
Function FtLMotorOff(nMotorNr)
 LOCAL nInputStatus:=0
 nInputStatus := FtLMotor(nMotorNr,ftiOff)
Return nInputStatus
//
Function FtLMotorOn(nMotorNr,nSpeed)
 LOCAL nInputStatus:=0
 nInputStatus := FtLMotorSpeed(nMotorNr,ftiOn,nSpeed)
Return nInputStatus
//
Function FTLAnalogScan()
 LOCAL lAnalogScan:=FALSE
 lAnalogScan := (DllCall(nUmFish30Handle,DLL_STDCALL,"csGetAnalogScan", ;
  iPortHandle) == 1)
 Return lAnalogScan
//
Function FTLAnalogScale()
 LOCAL nAnalogScale:=0
 nAnalogScale := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetLPTAnalog", ;
  iPortHandle)
 Return nAnalogScale
//
Function FTLLoopDelay()
 LOCAL nLoopDelay:=0
 nLoopDelay := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetLPTDelay", ;
  iPortHandle)
 Return nLoopDelay
//
Function FTLLptDelay()
 Return FTLLoopDelay()
//
Function FTLSlave()
 LOCAL lSlave:=FALSE
 lSlave := (DllCall(nUmFish30Handle,DLL_STDCALL,"csGetSlave",iPortHandle) ;
  == 1)
 Return lSlave
//
Function FTLExtension()
 Return FTLSlave()
//
Function FTLPollInterval() // in ms
 LOCAL nPollInterval:=0
 nPollInterval := DllCall(nUmFish30Handle,DLL_STDCALL,"csGetPollInterval", ;
  iPortHandle)
 Return nPollInterval  // in ms
//
Function FTLSetLamp(nLampNr,lOnOff)
 LOCAL nInputStatus:=0,nOnOff
 Default lOnOff to TRUE
 nOnOff := If(lOnOff,1,0)
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csSetLamp", ; 
   iPortHandle,nLampNr,nOnOff)
 Return nInputStatus
//
Function FtLOnNr(nO1toO16)
 Return FtLSetLamp(nO1toO16,TRUE)
//
Function FtLOffNr(nO1toO16)
 Return FtLSetLamp(nO1toO16,FALSE)
//
//
// RobMode
// 
//  motor    end input(Ex)   impuls input counter
//    1
//    2
//    3            5                 6
//    4            7                 8
//    5            9                10
//    6           11                12
//    7           13                14
//    8           15                16
//
// note assumes ftiLinks(0x01) directed to end input !
//
// example csRobMotors(iPortHandle,0x9,0xF6,0x10)
//
//  0x9 = 1001 = M1 = ftiLeft(01)
//               M2 = ftiRight(10)
//
//  0xF6 = 1111 0110 = speed M1 = 6 (0110)
//                     speed M2 = F = 15
//
//  0x10 = 0001 0000 = M1 in normal mode (0000)
//                     M2 in robomode (0001)
//
Function FTLRobMotors(nMotorStatus,nSpeedStatus,nModeStatus)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csRobMotors", ;
  iPortHandle,nMotorStatus,nSpeedStatus,nModeStatus)
 Return nInputStatus
//
Function FTLRobMotor(nMotorNr,nDirection,nSpeed,nCount)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish30Handle,DLL_STDCALL,"csRobMotor", ;
  iPortHandle,nMotorNr,nDirection,nSpeed,nCount)
 Return nInputStatus
//
Function FtLUsedLptPort()
 Return cLPT
//
Procedure FtLShowUsedIF()
 LOCAL cmsg:="FT interface uses "
 cmsg += cLPT + If(FTLSlave(),"(+slave)","")
 MsgBox(cmsg)
 Return 
//
// einde Ft30520.prg
//
